/*JavaScript PadLock AES SDK  */
function JS_AES_Class()
{	
	this.mode = "";
	this.key_length = 0;
	this.key = "";
	this.iv = "";
	this.flag = 0;
	
	/*
	Function : To test whether VIA ACE hardware is available
	Input: no 
	Output:
	 return:  1 : if VIA ACE hardware is available
		  0 : if VIA ACE hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_ace_available=function() {
	  	try{
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_ace_available();
			return ret;
	  	}catch(err){
			return err;
	  	}	
	};

	/*
	Function : To test whether VIA ACE2 hardware is available
	Input: no 
	Output:
	 return:  1 : if VIA ACE hardware is available
		  0 : if VIA ACE hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_ace2_available=function() {
		try{
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_ace2_available();
			return ret;
	  	}catch(err){
			return err;
	  	}
	};
	
	/*
	Function : To init a ace aes context by default value
	Input	 :
	Output   :
	return   : 1 : JS_padlock_ace_begin is successful
		   -1 : JS_padlock_ace_begin is failure by hardware 
	*/
	this.JS_padlock_ace_begin=function() {
		this.flag = this.JS_padlock_ace_available();
		if( this.flag != 1 ){
			return -1;
		}
		this.mode = "ECB";
		this.key_length = 128;
		this.key = "Hello,PadLock AES";
		return 1;
	};

	/*
	Function : To set key for aes
	Input	 : aes_key 	: original cipher key 
		   aes_key_len  : key length of the key must be 128/192/256 number
	Output   :
	return   : -1 : JS_padlock_ace_setkey is failure by hardware 
		   1 : JS_padlock_ace_setkey is successful
		   0 : JS_padlock_ace_setkey is failure by aes_key or aes_key_len incorrect
		   -2: aes_key_len incorrect
		   -3/-4: the length of aes_key incorrect
	*/
	this.JS_padlock_ace_setkey=function(aes_key,aes_key_len){
          	try{
			if(this.flag == 1){
				var myplugin=document.getElementById('embed');	
				if((isNaN(aes_key_len)) || ((aes_key_len!=128) && (aes_key_len!=192) && (aes_key_len!=256)))
				{
					this.flag = 0;
					return -2;
				}
				if(aes_key_len == 128)
				{
					if(aes_key.length > 16)
					{
						return -3;
					}
				}
				if(aes_key_len == 192)
				{
					if(aes_key.length > 24)
					{
						return -4;
					}
				}
	                	var ret = myplugin.aes_setkey(aes_key,aes_key_len);
				if(ret == 1 || ret == 2)
				{
					this.flag = 0;
					return 0;
				}else{
					this.key = ret;
                			this.key_length = aes_key_len;
					return ret;
				}
			}else{
				return -1;
			}
          	}catch(err){
                	alert(err);
          	}
	};

	/*
	Function : To set mode and iv for aes
	Input	 : aes_mode 	: aes mode must be ECB/CBC/CFB128/OFB128/CTR
		   aes_iv  : original iv of aes must be 16bit
	Output   :
	return   : -1 : JS_padlock_ace_setmodeiv is failure by hardware 
		   1 : JS_padlock_ace_setmodeiv is successful
		   0 : JS_padlock_ace_setmodeiv is failure by aes_mode or aes_iv incorrect
		   -2 : the length aes_iv incorrect
		   -3 : aes_mode incorrect
	*/
	this.JS_padlock_ace_setmodeiv=function(aes_mode,aes_iv){
          	try{         
			 if(this.flag == 1){
				var myplugin=document.getElementById('embed');
				if(!((aes_mode=="ECB") || (aes_mode=="CBC") || (aes_mode=="OFB128") || (aes_mode=="CFB128") || (aes_mode=="CTR")))
				{
					this.flag = 0;
					return -3;
				}
				if(aes_iv.length != 16)
				{
					this.flag = 0;
					return -2;
				}
                                var ret = myplugin.aes_setmodeiv(aes_mode,aes_iv);
				if( ret == 1)
				{
					this.flag = 0;
					return 0;
				}else{
					
                			this.mode = aes_mode;
					this.iv = ret;
					return ret;
				}

			}else{
				return -1;
			}
          	}catch(err){
                	alert(err);
          	}

        };

	/*
	Function :  To encrypt the text by VIA ACE hardware
	Input	 : text : the data of encrypt
	Output   :
	return   : -1 : JS_padlock_ace_encrypt is failure by hardware 
		   ret : the result of encrypt	
		   err : error info	
	*/
	this.JS_padlock_ace_encrypt=function(text){
                try{
			if( this.flag == 1){
                		var myplugin = document.getElementById('embed');
                		var ret = myplugin.aes_encrypt(text,this.mode,this.key_length,this.key,this.iv);
                        	return ret;
			}else{
				return -1;
			}
                }catch(err){
                        return err;
                }
        };

	/*
	Function :  To decrypt the text by VIA ACE hardware
	Input	 : text : the data of decrypt
	Output   :
	return   : -1 : JS_padlock_ace_decrypt is failure by hardware 
		   ret : the result of decrypt	
		   err : error info	
	*/
	this.JS_padlock_ace_decrypt=function(text)
        {
                try{
			if( this.flag == 1){	
                		var myplugin = document.getElementById('embed');
                		var ret = myplugin.aes_decrypt(text,this.mode,this.key_length,this.key,this.iv);
                        	return ret;
			}else{
				return -1;
			}
                }catch(err){
                        return err;
                }
        };

	/*
	Function : To clear a ace aes context by default value
	Input	 :
	Output   :
	return   : 	  
	*/
	this.JS_padlock_ace_close=function() {
                this.mode = null;
                this.key_length = null;
                this.key = null;
                this.flag = 0;
        };
	
}

/*JavaScript PadLock RNG SDK  */
function JS_RNG_Class()
{
	/*
	Function : To test whether VIA RNG hardware is available
	Input: no 
	Output:
	 return:  1 : if VIA RNG hardware is available
		  0 : if VIA RNG hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_rng_available=function() {
		  try{
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_rng_available();
			return ret;
	  	}catch(err){
		 	return err;
		}
	};
	
	/*
	Function : To generate random number by using VIA RNG hardware
	Input	 :  num : how long a random number need to be generated (must be number) 
	Output   : 	
	return   : -1  : JS_padlock_rng is failure by hardware 
		   err : error info
		   ret : random number
	*/
	this.JS_padlock_rng_long=function(num){
		 try{
			if(!isNaN(num) && num > 0 && this.JS_padlock_rng_available() == 1){
				var myplugin = document.getElementById('embed');
				var ret = myplugin.asm_rng_long(num);
				return ret;
			}else{
				return -1;
			}
	  	}catch(err){
			return err;
	  	}
	};
}

/*JavaScript PadLock SHA1 SDK  */
function JS_SHA1_Class()
{
	this.Key_length = 0;
	this.Hash = "";
	
	/*
	Function : To test whether VIA PHE hardware is available
	Input: no 
	Output:
	return:   1 : if VIA PHE hardware is available
		  0 : if VIA PHE hardware is unavailable
	      	  err:error info
	*/
	this.JS_padlock_phe_available=function(){
		try{
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_phe_available();
			return ret;
		}catch(err){
			return err;
	  	}
	};
	
	/*
	Function:padlock SHA1 process 
	Input:
	      string src
	      integer length 
	Output:
	      string dst
	return:   -1 : VIA PHE hardware not available
	      	  err: error info
		  -2 : src or length incorrect
	*/
	this.JS_padlock_sha1=function(src,length){
		try{
			if(this.JS_padlock_phe_available() == 1){
				if(length <= 0 || length != src.length)
				{
					return -2;
				}
				var myplugin = document.getElementById('embed');
				var dst = myplugin.phe_process_sha1(src,length);
				return dst;
			}else{
				return -1;
			}
		}catch(err){
			return err;
	  	}
	};
	
	/*
	Function : To test whether VIA padlock PHE partial hardware is available
	Input: no 
	Output:
	return:   1 : if VIA padlock PHE partial hardware is available
		  0 : if VIA padlock PHE partial hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_phe_partial_available=function(){
		try{
		        var myplugin = document.getElementById('embed');
		        var ret = myplugin.phe_partial_available();
			return ret;
		}catch(err){
			return err;
	  	}	
	};
	
	/*
	Function:SHA1 partial process init
	Input:
	       no 
	Output:
               no
	return:
		1：if VIA padlock PHE partial hardware available
		-1:if VIA padlock PHE partial hardware unavailable
		err:error info
	*/
	this.JS_padlock_phe_partial_sha1_init = function(){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				this.Key_length = 0;
				this.Hash = "67452301efcdab8998badcfe10325476c3d2e1f0";
				return 1;
			}else{
				return -1;
			}
		}catch(err){
			return err;
	  	}
	};
	
	/*
	Function:SHA1 partial process update
	Input:
	       string src
	       integer length 
	Output:
       		  this.hash
	return :
		-1:if VIA padlock PHE partial hardware unavailable
		-2:the length of src incorrect
		-3:length incorrect
		err:error info
	*/
	this.JS_padlock_phe_partial_sha1_update=function(src,length){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				if(src.length%64)
				{
					return -2;
				}
				if(length%64 || length != src.length)
				{
					return -3;
				}
				
				var myplugin = document.getElementById('embed');
				this.Hash = myplugin.phe_partial_update(src,this.Hash,length);
				this.Key_length = this.Key_length + length;
				return this.Hash;
			}else{
				return -1;
			}
		}catch(err){
			return err;
	  	}
	};
	
	/*
	Function:SHA1 partial process final
	Input:
	       string src
	       integer length 
	Output:
               this.Hash
	return:
		-2:src or length incorrect
		-1:if VIA padlock PHE partial hardware unavailable
		err : error info
	*/
	this.JS_padlock_phe_partial_sha1_final=function(src,length){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				if(length < 0 || length != src.length)
				{
					return -2;
				}
				var myplugin = document.getElementById('embed');
				this.Hash = myplugin.phe_partial_final(src,this.Hash,this.Key_length,length);
				return this.Hash;
			}else{
				return -1;
			}
		}catch(err){
			return err;
	  	}
	};
}

/*JavaScript PadLock SHA256 SDK  */
function JS_SHA256_Class()
{
	this.Key_length = 0;
	this.Hash = "";
	
	/*
	Function : To test whether VIA PHE hardware is available
	Input: no 
	Output:
	return:   1 : if VIA PHE hardware is available
		  0 : if VIA PHE hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_phe_available=function(){
		try{
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_phe_available();
			return ret;
		}catch(err){
			return err;
		  }
	};
	
	/*
	Function:padlock SHA256 process 
	Input:
	      string src
	      integer length 
	Output:
	      string dst 
	return:   -1 : if VIA PHE hardware unavailable
	      	  err : error info
		  -2 : src or length incorrect
	*/
	this.JS_padlock_sha256=function(src,length){
		try{
			if(this.JS_padlock_phe_available() == 1){
				if(length <= 0 || length != src.length)
				{
					return -2;
				}
			
				var myplugin = document.getElementById('embed');
				var dst = myplugin.phe_process_sha256(src,length);
				return dst;
			}else{
				return -1;
			}
		}catch(err){
			return err;
		}
	};
	
	/*
	Function : To test whether VIA padlock PHE partial hardware is available
	Input: no 
	Output:
	return:   1 : if VIA padlock PHE partial hardware is available
		  0 : if VIA padlock PHE partial hardware is unavailable
	      	  err : error info
	*/	
	this.JS_padlock_phe_partial_available=function(){
		try{
		        var myplugin = document.getElementById('embed');
		        var ret = myplugin.phe_partial_available();
			return ret;
		}catch(err){
			return err;
		}	
	};
	
	/*
	Function:SHA1 partial process init
	Input:
	       no 
	Output:
               no
	return:
		1：if VIA padlock PHE partial hardware available
		-1:if VIA padlock PHE partial hardware unavailable
		err : error info
	*/
	this.JS_padlock_phe_partial_sha256_init=function(){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				this.Key_length = 0;
				this.Hash = "6a09e667bb67ae853c6ef372a54ff53a510e527f9b05688c1f83d9ab5be0cd19";
				return 0;
			}else{
				return -1;
			}
		}catch(err){
			return err;
		}
	};
	
	/*
	Function:SHA1 partial process update
	Input:
	       string src
	       integer length 
	Output:
       		  this.hash
	return :
		-1:if VIA padlock PHE partial hardware unavailable
		-2:the length of src incorrect
		-3:length incorrect
		err : error info
	*/
	this.JS_padlock_phe_partial_sha256_update=function(src,length){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				if(src.length%64)
				{
					return -2;
				}
				if(length%64 || length != src.length)
				{
					return -3;
				}
				
				var myplugin = document.getElementById('embed');
				this.Hash = myplugin.phe_partial_256_update(src,this.Hash,length);
				this.Key_length = this.Key_length + length;
				return this.Hash;
			}else{
				return -1;
			}
		}catch(err){
			return err;
		}
	};
	
	/*
	Function:SHA1 partial process final
	Input:
	       string src
	       integer length 
	Output:
               this.Hash
	return:
		-2:src or length incorrect
		-1:if VIA padlock PHE partial hardware unavailable
		err : error info
	*/
	this.JS_padlock_phe_partial_sha256_final=function(src,length){
		try{
			if(this.JS_padlock_phe_partial_available() == 1){
				if(length < 0 || length!=src.length)
				{
					return -2;
				}
				var myplugin = document.getElementById('embed');
				this.Hash = myplugin.phe_partial_256_final(src,this.Hash,this.Key_length,length);
				return this.Hash;
			}else{
				return -1;
			}
		}catch(err){
			return err;
		}
	};
}

/*JavaScript PadLock PMM SDK  */
function JS_PMM_Class()
{
	this.OSbits=function(){
		var x=navigator.userAgent.match(/x86_64|Win64|WOW64/) || navigator.cpuClass=='x64' ?'x64':'x86'
		return x;
	}

	/*
	Function : To test whether VIA PMM hardware is available
	Input: no 
	Output:
	 return:  1 : if VIA PMM hardware is available
		  0 : if VIA PMM hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_pmm_available=function() {
		try{
			var x=this.OSbits();
			if(x == 'x64'){
				return 0;
			}
			else{
				var myplugin = document.getElementById('embed');
				var ret = myplugin.padlock_pmm_available();
		
				return ret;
			}
			}catch(err){
				return err;
			}
	};

	/*
	Function : To test whether VIA PMM2 hardware is available
	Input: no 
	Output:
	return:  1 : if VIA PMM2 hardware is available
		  0 : if VIA PMM2 hardware is unavailable
	      	  err : error info
	*/
	this.JS_padlock_pmm2_available=function() {
		try{
			var x=this.OSbits();
			if(x == 'x64'){
				return 0;
			}
			else{
				var myplugin = document.getElementById('embed');
				var ret = myplugin.padlock_pmm2_available();
				return ret;
			}
		  }catch(err){
			return err;
		  }
	};
	
	/*
	Function : To implement Montgomery Multiplication algorithm			
	Input	 : A :  big integer A
		   B :  big integer B
		   M :  big integer M, modulus
	           ndigits : the length of big integer A, B, M, T, in digits(32bit). 
	Output	 :
	return	 : -1  : JS_PadLock_pmm by hardware 
		   ret : result of count
		   err : error info
		   -2  : the length of A B M is not equal, and not be multiples of 128
		   -3  : ndigits incorrect
	*/
	this.JS_PadLock_pmm=function(A,B,M,ndigits) {
		try{
			if(this.JS_padlock_pmm_available() == 1){
			if(A.length!=B.length || A.length!=M.length || A.length%128!=0)
			{
				return -2;
			}
			if(ndigits%16 != 0 || ndigits*8 != A.length)
			{
				return -3;
			}
			var myplugin = document.getElementById('embed');
			var ret = myplugin.padlock_pmm(A,B,M,ndigits);
			return ret;
			}else{
				return -1;
			}
		  }catch(err){
			return err;
		  }
	};
}
